/* ************************************************************************* */
/* Diag_16.c                                                                 */
/* DS Software Authorization SDK 4.3                                         */
/* Disk 1 - Dos Object & Source Files                                        */  
/* 02/016/98                                                                  */
/*                                                                           */
/* Copyright (C) 1996 Dallas Semiconductor Corporation.                      */
/* All rights Reserved. Printed in U.S.A.                                    */
/* This software is protected by copyright laws of                           */
/* the United States and of foreign countries.                               */
/* This material may also be protected by patent laws of the United States   */
/* and of foreign countries.                                                 */
/* This software is furnished under a license agreement and/or a             */
/* nondisclosure agreement and may only be used or copied in accordance      */
/* with the terms of those agreements.                                       */
/* The mere transfer of this software does not imply any licenses            */
/* of trade secrets, proprietary technology, copyrights, patents,            */
/* trademarks, maskwork rights, or any other form of intellectual            */
/* property whatsoever. Dallas Semiconductor retains all ownership rights.   */
/*                                                                           */
/* Additional Notes:                                                         */
/*                                                                           */
/*    This software is intended for use with DS1410D and DS1410E and later   */
/*    model parallel port adapters only.                                     */
/*                                                                           */
/*    Use of groundtest() is no longer necessary.  Although code support     */
/*    will continue to be backwards compatible, its use in this example has  */
/*    been dropped in order to provide a clearer example for developers.     */ 
/*                                                                           */ 
/*    Code has been slightly modified in order to sense and make use of the  */
/*    presence of the DS1994 in addition to the DS1425.                      */
/*    (Commands for each are the same, only the family code differs.)        */
/*                                                                           */ 
/* ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

#define TRUE  1
#define FALSE 0

#define READSCRATCH 0x69         /* Read scratchpad command byte */ 
#define SP_SA       0xC0         /* Field specifier / starting address */ 

#define DS1425FamilyCode 0x82
#define DS1991FamilyCode 0x02
#define DS1996FamilyCode 0x0C

typedef unsigned char uchar;
typedef unsigned int uint;

uchar far pascal dowcheck(void); /* Access system functions */
uchar far pascal keyopen(void);
uchar far pascal keyclose(void);
uchar far pascal setup(uchar);
uchar far pascal next(void);
uchar far pascal first(void);
uchar far pascal access(void);
uchar far pascal gndtest(void);
uchar far pascal databyte(uchar);
uchar far pascal OverdriveOn(void);
void  far pascal OverdriveOff(void);
uchar far * far pascal romdata(void);

uint  LptAddress[3]   = { 0x378, 0x278, 0x3BC };
uchar RetVal[2][7] = { "FALSE", "TRUE" };

void main(int argc, char *argv[])
{ /* MAIN PROGRAM */

    uchar pn = 0;                 /* Parallel port number */
    uchar far *addr;              /* Holds ROM data pointer */
    uchar NoButtons;
    uchar FirstSuccess, NextSuccess, ButtonCount;
    uchar MemoryString[32];
    unsigned long TotalButtonsFound=0;
    int   i, j;
    int   loop_num; 
    int   read_num;
    
    /* Grab read_num parameter off command line.                   */
    /* Variable read_num is the number of times to iterate through */
    /* search for convenient reliability testing.                  */
    if (argc > 1)                                        
      read_num = atoi(argv[1]);
    else
      read_num = 1;
    
    
    if (dowcheck())                     
    { /* Check whether if device driver is installed. */
    
        while ((++pn < 4) && setup(pn))                   
        { /* Iterate through all ports. */
              
            keyopen();
                        
            /* Initialize loop counter. */
            loop_num = 0; NoButtons = FALSE;     
            while ((loop_num++ < read_num) && !NoButtons)
            { /* do read_num iterations for reliability testing. */
            
               /* Look for first part on bus */
               FirstSuccess = first();
               printf("<first returned %s>\n", RetVal[FirstSuccess]);
            
               if (FirstSuccess)                  
               { /* At least one iButton found on port. */
               
                  ButtonCount = 0;
            
                  do 
                  { /* While still more iButtons. */ 
                  
                     ++ButtonCount;
                     ++TotalButtonsFound;          
                     
                     /* Release control of port between reliability iterations. */
                     /* (Not necessary, but very good practice.)                */
                     keyclose(); keyopen();
                                                                               
                     printf("\nRom Data: ");     
                         
                     /* Get pointer to ROM data */
                     addr = romdata();           
                     /* Display ROM data */
                     for (i = 0; i < 8; i++)            
                        printf("%2.2X ", *(addr + i));
                  
                     
                     if ((*addr == DS1425FamilyCode) && access()) 
                     { /* Is iButton a DS1425? */
                     
                        /* Send read scratch command */
                        databyte(READSCRATCH); 
                        /* Send field / starting address */
                        databyte(SP_SA);   
                        /* Send confirmation byte */
                        databyte((uchar) ~SP_SA);         
                  
                        printf("\nScratchpad data:\n");
                  
                        /* Read scratchpad data */
                        for (i = 0; i < 64; i++) 
                           printf("%c", databyte(0xFF));
                           
                     } /* Is iButton a DS1425? */
                     else if ((*addr == DS1991FamilyCode) && access()) 
                     { /* Is iButton a DS1991? */
                     
                        /* Send read scratch command */
                        databyte(READSCRATCH); 
                        /* Send field / starting address */
                        databyte(SP_SA);   
                        /* Send confirmation byte */
                        databyte((uchar) ~SP_SA);         
                  
                        printf("\nScratchpad data:\n");
                  
                        /* Read scratchpad data */
                        for (i = 0; i < 64; i++) 
                           printf("%c", databyte(0xFF));
                           
                     } /* Is iButton a DS1991? */
                     else if (*addr == DS1996FamilyCode)
                     { /* Is iButton a DS1996? */
                     
                        /* Switch to overdrive. */
                        if (OverdriveOn())
                        { /* Overdrive Capable. */

                           if (access());
                           {
                              printf("\nWriting memory contents\n");
            
                              memset(MemoryString, 0x20, 32);
                   
                              for (i = 0; i < 254; i++)
                              {
                                 access();
            
                                 databyte(0x0F);
                                 databyte((uchar) (i << 5));
                                 databyte((uchar) (i >> 3));
                                          
                                 sprintf(MemoryString, 
                                         "%s%d", 
                                         "This is secure memory page #",
                                         i);
            
                                 for (j = 0; j < 32; j++)
                                 {
                                    /* printf("%c", MemoryString[j]); */
                                    databyte(MemoryString[j]);
                                 }
            
                                 access();
            
                                 databyte(0x55);
                                 databyte((uchar) (i << 5));
                                 databyte((uchar) (i >> 3));
                                 databyte(0x1F);
                              }
            
                              printf("Reading memory contents\n");
            
                              access();
            
                              databyte(0xF0);
                              databyte(0);
                              databyte(0);
            
                              for (i = 0; i < 254; i++) 
                              {
                                 for (j = 0; j < 32; j++)
                                 {
                                    printf("<%2.2X>", databyte(0xFF));
                                 }
            
                                 printf("\n");
                              }                     
                              
                           } /* Access successful. */ 
            
                           /* Turn overdrive mode off. */
                           OverdriveOff();       
                           
                        } /* Overdrive Capable. */
                        else
                           printf("Unable to switch to overdrive mode\n");
                     } /* Is iButton a DS1996? */
                                   
                     printf("\n");
            
                     NextSuccess = next();
                     printf("<next returned %s>\n", RetVal[NextSuccess]);
                     
                  } /* While still more iButtons. */
                  while (NextSuccess);  /* Look at all parts on port. */
            
                  printf("\nFound %d button(s) on port address %2.2X\n",
                         ButtonCount,
                         LptAddress[pn - 1]);                
                         
               } /* At least one iButton found on port. */
               else 
               {
                  NoButtons = TRUE;
                  printf("\nNo buttons found on port address %2.2X\n", 
                          LptAddress[pn - 1]);
               }                                                  
               
            } /* do read_num iterations for reliability testing. */
            
            /* Release control of port. */
            keyclose();                   
                             
        } /* Iterate through all ports. */                                                 
      
    } /* Check whether if device driver is installed. */
    else
    { /* Call to dowcheck() failed. */
    
      printf("DOW device driver not installed.\n");
      
    } /* Call to dowcheck() failed. */
    
    printf("Number of total buttons found = %lu", TotalButtonsFound);
       
} /* MAIN PROGRAM */


