/* ************************************************************************* */
/* ProgPwd.c                                                                 */
/* DS Software Authorization SDK 4.3                                         */
/* Disk 1 - Dos Object & Source Files                                        */
/* 02/06/98                                                                  */
/*                                                                           */
/* Copyright (C) 1996 Dallas Semiconductor Corporation.                      */
/* All rights Reserved. Printed in U.S.A.                                    */
/* This software is protected by copyright laws of the United States and of  */
/* foreign countries.                                                        */
/* This software is furnished under a license agreement and/or a             */
/* nondisclosure agreement and may only be used or copied in accordance      */
/* with the terms of those agreements.                                       */
/* The mere transfer of this software does not imply any licenses            */
/* of trade secrets, proprietary technology, copyrights, patents,            */
/* trademarks, maskwork rights, or any other form of intellectual            */
/* property whatsoever.  Dallas Semiconductor retains all ownership rights.  */
/*                                                                           */
/* Additional Notes:                                                         */
/*                                                                           */
/*    This code was compiled using Boreland 4.5 along with the Turbo         */
/*    Assembler.  Use of other compilers may require slight modification.    */
/*                                                                           */
/*    To make project, link with <Sauth400.c>, or <Sauth400.asm>.            */
/*                                                                           */
/*    This software is intended for use with DS1410D and DS1410E and later   */
/*    model parallel port adapters only.                                     */
/*                                                                           */
/*    This program searches for a DS1991. If found it sets the reprograms    */
/*    secure subkey 0 to user specified data.                                */
/*                                                                           */
/* ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <mem.h>

#define TRUE  1
#define FALSE 0

typedef unsigned char  uchar;
typedef unsigned long  ulong;
typedef unsigned short ushort;

typedef struct _SubKeyStruct
{
   uchar ID[8];
   uchar Password[8];
   uchar SecureData[48];
}
SubKeyStruct, *PSubKeyStruct;

//
//   SecureTMEX (access system) function prototypes
//
uchar far pascal dowcheck(void);  
uchar far pascal keyopen(void);
uchar far pascal keyclose(void);
uchar far pascal setup(uchar);
uchar far pascal next(void);
uchar far pascal first(void);
uchar far pascal access(void);
uchar far pascal gndtest(void);
uchar far pascal databyte(uchar);
uchar far * far pascal romdata(void);

uchar SetNewPassword(PSubKeyStruct KeyData, uchar KeyNum)
{
   uchar OldID[8];
   int   i;

   // Reset/Address button
   if (access())
   {
      // Send reprogram subkey command byte
      databyte(0x5A);
      // Specify key number for operation
      databyte(KeyNum << 6);
      // Ones complement confirmation byte
      databyte(~(KeyNum << 6));

      // Read old ID field
      for (i = 0; i < 8; i++)
         OldID[i] = databyte(0xFF);
      
      // Send back old ID field
      for (i = 0; i < 8; i++)
         databyte(OldID[i]);

      // Send new ID field 
      for (i = 0; i < 8; i++)
         databyte(KeyData->ID[i]);

      // Send new ID password 
      for (i = 0; i < 8; i++)
         databyte(KeyData->Password[i]);

      return TRUE;
   }

   return FALSE;
}

uchar WriteNewSubKeyData(PSubKeyStruct KeyData, uchar KeyNum)
{
   int   i;

   // Reset/Address button
   if (access())
   {
      // Send write subkey command byte
      databyte(0x99);
      // Specify key number + starting address
      databyte((KeyNum << 6) + 0x10);
      // Ones complement confirmation byte
      databyte(~((KeyNum << 6) + 0x10));

      // We need to read the ID, but we don't care about results here
      for (i = 0; i < 8; i++)
         databyte(0xFF);
         
      // Send password of the secure subkey
      for (i = 0; i < 8; i++)
         databyte(KeyData->Password[i]);

      // Send new secure data
      for (i = 0; i < 48; i++)
         databyte(KeyData->SecureData[i]);
   
      return TRUE;
   }

   return FALSE;
}

uchar ReadSubKeyData(PSubKeyStruct KeyData, uchar KeyNum)
{
   int i;

   // Reset/Address button
   if (access())
   {
      // Send read subkey command byte
      databyte(0x66);
      // Specify key number + starting address
      databyte((KeyNum << 6) + 0x10);
      // Ones complement confirmation byte
      databyte(~((KeyNum << 6) + 0x10));

      // Read ID field 
      for (i = 0; i < 8; i++)
         KeyData->ID[i] = databyte(0xFF);
         
      // Send password of the secure subkey
      for (i = 0; i < 8; i++)
         databyte(KeyData->Password[i]);

      // Read secure data
      for (i = 0; i < 48; i++)
         KeyData->SecureData[i] = databyte(0xFF);

      return TRUE;
   }

   return FALSE;
}

void Intro(void)
{
   printf("\n               ProgPwd.c - reprograms subkey #0 ID, password");
   printf("\n               and writes new secure data in the subkey. After");
   printf("\n               the reprogramming it reads back the ID and");
   printf("\n               secure data using the user supplied password"); 
   printf("\n               and displays the results. Note that this simple");
   printf("\n               program only uses scanf for input. Don't use"); 
   printf("\n               spaces\n\n"); 
}

void main(void)
{
   SubKeyStruct Key0;
   ushort       pa_vals[3] = { 0x378, 0x278, 0x3BC };
   uchar        pn = 0;       // Parallel port number
   uchar        far *addr;    // Holds ROM data pointer 
   uchar        Found1991;
   uchar        TempStr[80];
   int          i;

   Intro();

   // See if device driver is installed 
   if (dowcheck())                
   {
      // Search all ports
      while ((++pn < 4) && setup(pn))                   
      {  // Gain control of port access
         keyopen();
         // Make sure we can talk to DS1410D 
         if (gndtest())               
         {
            // Look for first part on bus
            if (first())                
            {
               Found1991 = FALSE;

               do
               {
                  // Release control of port between passes 
                  keyclose();  
                  // Regain control of port
                  keyopen();
   
                  // Get pointer to ROM data 
                  addr = romdata();           

                  // Is this a DS1991 password button
                  if (((*addr == 0x82) || (*addr == 0x02)) && access())
                  {
                     Found1991 = TRUE;

                     printf("\nDS1991/DS1425 Rom Data: ");
                     for (i = 0; i < 8; i++)
                        printf("%2.2X ", *(addr + i));
                    
                     memset(TempStr, 0, 80);
                     printf("\n\nEnter a new id field (<= 8 bytes) :");
                     scanf("%s", TempStr);
                     memcpy(Key0.ID, TempStr, 8);

                     memset(TempStr, 0, 80);
                     printf("Enter new password (<= 8 bytes)   :");
                     scanf("%s", TempStr);
                     memcpy(Key0.Password, TempStr, 8);

                     memset(TempStr, 0, 80);
                     printf("Enter secure data (<= 48 bytes)   :");
                     scanf("%s", TempStr);
                     memcpy(Key0.SecureData, TempStr, 48);

                     // Reprogram password and ID of secure subkey #0
                     if (!SetNewPassword(&Key0, 0))
                     {
                        printf("Unable to set new password.\n");
                        exit(1);
                     }

                     // Write new secure data to secure subkey #0
                     if (!WriteNewSubKeyData(&Key0, 0))
                     {
                        printf("Unable to write new secure data.\n");
                        exit(1);
                     }

                     // Clear ID field 
                     memset(Key0.ID, 0, 8);
                     // Clear secure data field 
                     memset(Key0.SecureData, 0, 48);

                     // Read back new secure data from secure subkey #0
                     if (!ReadSubKeyData(&Key0, 0))
                     {
                        printf("Unable to read back secure data.\n");
                        exit(1);
                     }
                     else
                     {
                        printf("\n\nNew ID field    :");
                        // Display contents of ID field
                        for (i = 0; i < 8; i++)
                           if (Key0.ID[i] > 31)
                              putchar(Key0.ID[i]);

                        printf("\nNew secure data :");
                        // Display new secure data
                        for (i = 0; i < 48; i++)
                           if (Key0.SecureData[i] > 31)
                              putchar(Key0.SecureData[i]);

                        printf("\n");
                     }
                  }
               }
               while (next()); // Look at all parts on port

               if (!Found1991)
                  printf("Unable to find any DS1991-Compatible devices.\n");
            }
            else 
            {
               printf("Unable to find DS1410D on port address %4.4X\n", 
                      pa_vals[pn - 1]);
            }
         } 
         else 
         {
            printf("Unable to find DS1410D on port address %4.4X\n", 
                   pa_vals[pn - 1]);
         }
         // Release control of access system 
         keyclose();
      }

   }
   else
   {
      // Not meaningful in DOS, used for portability
      printf("DOW device driver not installed.\n");
   }
}


