{ Tsample.pas                                                               }
{ DS Software Authorization SDK 4.3                                         }
{ Disk 1 - Dos Object & Source Files                                        }
{ 02/06/98                                                                  }
{                                                                           }
{ Copyright (C) 1996 Dallas Semiconductor Corporation.  All rights          }
{ Reserved. Printed in U.S.A.  This software is protected by copyright laws }
{ of the United States and of foreign countries.  This software is          }
{ furnished under a license agreement and/or a nondisclosure agreement and  }
{ may only be used or copied in accordance with the terms of those          }
{ agreements.  The mere transfer of this software does not imply any        }
{ licenses of trade secrets, proprietary technology, copyrights, patents,   }
{ trademarks, maskwork rights, or any other form of intellectual            }
{ property whatsoever. Dallas Semiconductor retains all ownership rights.   }
{                                                                           }
{ Additional Notes:                                                         }
{                                                                           }
{      This code was compiled using Borland's Turbo Pascal 6.0 compiler.    }
{ Use of other compilers may require some changes to the source code.       }
{                                                                           }
{      This code is intended for use with Dallas Semiconductor parts        }
{ DS1410D and DS1410E parallel port adapters.                               }
{                                                                           }

Program Tsample;

Uses DOS, CRT, SauthP;

Procedure Center(X: String);

Begin { Proc Center }

  GotoXY(Succ((80-Length(X)) shr 1), WhereY);
  WriteLn(X);

End;  { Proc Center }

Function Hex(X: Byte): String;
{                                                                           }
{  This function returns the hexadecimal representation string              }
{  of the byte passed to it.                                                }
{                                                                           }

Var
  S : String[2];
  I, J : Byte;

Begin { Func Hex }

  S := '';
  For I := 1 to 2 do Begin
     J := X and $F;
     X := X shr 4;
     If J > 9 then Inc(J, 7);
     S := Char(J + $30) + S
  End;
  Hex := S;

End;  { Func Hex }

Function LZ(X: Byte): String;

Var
  S: String[2];

Begin { Func LZ }

  S := '';
  If X < 10 Then S := Char($30) + Char($30 + X) Else Str(X, S);
  LZ := S;

End;  { Func LZ }

Procedure ComBar;
Begin { Proc ComBar }

  Window(1, 25, 80, 25); Textcolor(Red); TextBackGround(LightGray); ClrScr;

End;  { Proc ComBar }

Procedure Banner;

Begin { Proc Banner }

  Window(1, 1, 80, 1); Textcolor(Red); TextBackGround(LightGray); ClrScr;
  Write('   DALLAS SEMICONDUCTOR       DS1994 DEMONSTRATION PROGRAM      SDK Version 4.3');

End;  { Proc Banner }

Procedure TextWin;

Begin { TextWin }

  Window(1, 2, 80, 24); TextBackGround(Blue); TextColor(Yellow);

End;  { TextWin }

Procedure InitCrt;

Begin { InitCrt }

  ComBar;
  Write('    PGUP, PGDN => CHANGE PAGE         F1 => LEGEND              ESC => QUIT');
  Banner;
  TextWin;
  ClrScr;

End;  { InitCrt }

Var
  Ch: Char;

Procedure Help;

Begin { Proc Help }

  Window(1, 1, 80, 24);
  ClrScr;
  GotoXY(1, 4);
  Center('');
  Center('  KEYSTROKE                  FUNCTION                  ');
  Center('');
  Center('         ESC                    QUIT                   ');
  Center('        PGUP  INCREMENT SECURE MEMORY PAGE DISPLAYED   ');
  Center('        PGDN  DECREMENT SECURE MEMORY PAGE DISPLAYED   ');
  Center('        RARR  INCREMENT DATA FIELD (USED IN CLOCK PAGE)');
  Center('        LARR  DECREMENT DATA FIELD (USED IN CLOCK PAGE)');
  Center('        UARR  NEXT DS1994-COMPATIBLE DEVICE ON BUS     ');
  Center('        DARR  PREVIOUS DS1994-COMPATIBLE DEVICE ON BUS ');
  Center('       ALT-T  TOGGLE BETWEEN CLOCK AND MEMORY SCREENS  ');
  Center('');
  GotoXY(28, 20);
  Write('PRESS ANY KEY TO CONTINUE');
  Repeat Until Keypressed;
  Ch := Readkey;

End;  { Proc Help }

Procedure Warning;

Begin { Proc Warning }

  Window(1, 1, 80, 24);
  ClrScr;
  GotoXY(1, 1);
  Center('WARNING !!!');
  Center('');
  Writeln;
  Center('   If any of the bits 0 - 2 (WPR, WPE, WPC) in byte 201H');
  Center('are set to ON (1 binary) bits 0 - 4 (WPR, WPE, WPC, RO, ');
  Center('OSC) are PERMANENTLY and IRREVERSIBLY write protected.  ');
  Center('Also if bit 2 (WPC) is set to ON, bit 7 (DSEL) is write ');
  Center('protected.                                              ');
  Center('   If any of bits 0 - 2 are set and bit 3 (RO) is OFF,  ');
  Center('once an alarm occurs only 1''s can be read from the part.');
  Center('The bits mentioned above are defined as follows:        ');
  Writeln;
  Center('          WPR => Write protect the real time clock      ');
  Center('          WPE => Write protect the elapse timer         ');
  Center('          WPC => Write protect the cycle counter        ');
  Center('          RO  => Read only                              ');
  Center('          OSC => Delay select                           ');
  Writeln;
  Center('   If you do not fully understand the functions of these');
  Center('bits please refer to the DS0607 Developers Kit.         ');
  GotoXY(25, 23);
  Write('PRESS ANY KEY TO CONTINUE');
  Repeat Until Keypressed;
  Ch := Readkey;

End;  { Proc Warning }

Const
  DM : Array [1..12] of Word = ( 0, 31, 59, 90, 120, 151, 181, 212,
                                 243, 273, 304, 334 );

Type
  PageType = Array[0..31] of Byte;
  AuthType = Array[0..2] of Byte;

Var
  I, J, X, H : Byte;
  Tick       : Boolean;
  L          : LongInt;
  AuthCode   : AuthType;
  PageData   : PageType;

Function Read_S_Mem(PageNo: Byte; Var PageData: PageType): Boolean;

Begin { Func Read_S_Mem }

  Read_S_Mem := False;
  If Access Then Begin     { Initialize communication with selected DS1994 }
    Tick := True;
    X := DataByte($F0);                  { Send read secure memory command }
    X := DataByte(($20 * PageNo) And $FF);    { Send LSB of target address }
    X := DataByte((($20 * PageNo) And $0300) Shr 8);            { Send MSB }
    For H := 0 to 31 do PageData[H] := DataByte($FF);     { Read SMEM page }
    Read_S_Mem := True;
  End;

End;  { Func Read_S_Mem }

Function WriteScratch(PageNo: Byte; Var PageData: PageType): Boolean;

Begin { WriteScratch }

  WriteScratch := False;
  If Access Then Begin     { Initialize communication with selected DS1994 }
    X := DataByte($0F);                    { Send write scratchpad command }
    X := DataByte(($20 * PageNo) And $FF); { Send high byte of target addr }
    X := DataByte((($20 * PageNo) And $0300) Shr 8); { Send low byte of TA }
    For H := 0 to 31 do X := DataByte(PageData[H]);{ Write scratchpad data }
    WriteScratch := True;
  End;

End; { WriteScratch }

Function ReadScratch(Var PageData: PageType; Var AuthCode: AuthType): Boolean;

Begin { ReadScratch }

  ReadScratch := False;
  If Access Then Begin     { Initialize communication with selected DS1994 }
    X := DataByte($AA);                     { Send read scratchpad command }
    For H := 0 to 2 Do AuthCode[H] := DataByte($FF);     { Read auth. code }
    For H := 0 to 31 Do PageData[H] := DataByte($FF);    { Read scratchpad }
    ReadScratch := True;
  End;

End;  { ReadScratch }

Function CopyScratch(Var AuthCode: AuthType): Boolean;

Begin { Func CopyScratch }

  CopyScratch := False;
  If Access Then
  Begin                    { Initialize communication with selected DS1994 }
    X := DataByte($55);                     { Send copy scratchpad command }
    For H := 0 to 2 Do X := DataByte(AuthCode[H]);{Send authorization code }
    CopyScratch := True;
  End;

End;  { Func CopyScratch }

Const
  XM : Array[0..4] of Byte = ( 38, 9, 9, 9, 9 );      { Screen coordinates }
  YM : Array[0..4] of Byte = ( 4, 6, 8, 13, 20 );

Type
  Dat = Record
    Year : Word;
    Day  : Byte;
    Mon  : Byte;
  End;
  Tim = Record
    Min  : Byte;
    Hour : Byte;
    Hund : Byte;
    Sec  : Byte;
  End;

Var
  PageNo  : Byte;
  Da_Date : Dat;
  Ti_Time : Tim;
  Xv, Yv     : LongInt;
  Iv, Jv, Sv : Word;
  Bv         : Byte;

Function DTU(Var D: Dat; Var T:Tim): LongInt;

Begin { Func DTU }

  Sv := Da_Date.Year - 1968;
  If (Da_Date.Mon > 2) And ((Sv and 3) = 0) Then Bv := 1 Else BV := 0;
  Xv := 365 * (Sv - 2) + (Sv - 1) div 4 + DM[Da_Date.Mon] + Da_Date.Day + Bv - 1;
  Xv := 86400 * Xv + Ti_Time.Sec + 60*(LongInt(Ti_Time.Min) + 60*(LongInt(Ti_Time.Hour)));
  DTU := Xv;

End;  { Func DTU }


Procedure UTD(L: LongInt; Var D: Dat; Var T:Tim);

Begin { Func UTD }

  If L < 0 Then Exit;
  Xv := L div 86400;                   { Get number of days since 1/1/1970 }
  Yv := L - Xv * 86400;             { Number of seconds in fraction of day }
  Xv := 4 * (Xv + 731); Da_Date.Year := Xv div 1461;
  Iv := (Xv - 1461 * Da_Date.Year) div 4; Da_Date.Mon := 13;
  Repeat
     Dec(Da_Date.Mon);
     Jv := DM[Da_Date.Mon] +
           Byte((Da_Date.Mon > 2) and ((Da_Date.Year and 3) = 0));
  Until Iv >= Jv;
  Da_Date.Day := Iv - Jv + 1; Da_Date.Year := Da_Date.Year + 1968;
  Ti_Time.Hour := Yv Div 3600;
  Ti_Time.Min := (Yv - LongInt(Ti_Time.Hour) * 3600) div 60;
  Ti_Time.Sec := Yv - LongInt(Ti_Time.Min) * 60 - LongInt(Ti_Time.Hour) * 3600;
End;  { Func UTD }

Procedure Printit(FN: Byte);

Begin { Proc Printit }

  GotoXY(1, 10); Center('SCRATCH PAGE'); Center('');
  If FN = 4 Then Begin
    GotoXY(30, 17); Writeln('SECURE MEMORY PAGE # ', LZ(PageNo));
    GotoXY(30, 18); Writeln('');
  End;
  GotoXY(XM[FN], YM[FN]);
  For I := 0 To 15 Do Write(Hex(PageData[I]), ' ');
  GotoXY(XM[FN] + 49, YM[FN]);
  For I := 0 To 15 Do If (PageData[I] < $80) and (PageData[i] > $19) Then
      Write(Char(PageData[i])) Else Write('.');
  GotoXY(XM[FN], YM[FN] + 1);
  For I := 16 to 31 Do Write(Hex(PageData[I]), ' ');
  GotoXY(XM[FN] + 49, YM[FN] + 1);
  For I := 16 to 31 Do If (PageData[I] < $80) and (PageData[i] > $19) Then
      Write(Char(PageData[I])) Else Write('.');

End;  { Proc Printit }

Type
  RomAry = Array[1..8] of Byte;

Var
  RomPtr : ^RomAry;
  RomDta : RomAry;
  FindNo : Byte;
  CurrNo : Byte;
  Alarmf : Byte;      { Can have value from 0 to 7, depends on alarm states }

Function DS1994: Boolean;

Begin { Func DS1994 }

  RomPtr := RomData;
  RomDta := RomPtr^;
  If ((RomDta[1] <> $84) OR (RomDta[1] <> $04)) Then
     DS1994 := True
  Else
     DS1994 := False;

End;  { Func DS1994 }

Procedure CloseIt;

Begin { Proc CloseIt }

  Window(1, 1, 80, 25);
  ClrScr;
  WriteLn('At least one DS1994-Compatible device must be connected!');
  Halt;

End;  { Proc CloseIt }

Procedure FindAll;

Begin { Proc FindAll }

  FindNo := 0;
  If First Then
  Begin
    If DS1994 Then
       Inc(FindNo);
    While Next Do
       If DS1994 Then
          Inc(FindNo);
  End;

End;  { Proc FindAll }

Procedure GetLast;

Var
  Fail : Boolean;

Begin { GetLast }

  FindAll;
  Fail := False;
  While Next Do;
  If CurrNo = 1 Then CurrNo := FindNo Else Dec(CurrNo);
  For I := 1 To CurrNo Do If Next Then;
  If DS1994 Then Exit;
  While Not Fail Do Begin
    If First Then Begin
      If DS1994 Then Begin
        If CurrNo = 1 Then CurrNo := FindNo Else Dec(CurrNo);
        Exit;
      End;
      While Next Do If DS1994 Then Begin
        If CurrNo = 1 Then CurrNo := FindNo Else Dec(CurrNo);
        Exit;
      End;
    End;
    Fail := True;
  End;
  CloseIt;

End;  { GetLast }

Procedure GetNext;

Var
  Fail, TR : Boolean;
  OnePass  : Byte;

Begin { Proc GetNext }

  Fail := False;
  OnePass := 0;
  While Not Fail Do Begin
    TR := Next;
    If TR and DS1994 Then Begin
      If OnePass = 1 Then CurrNo := 1 Else Inc(CurrNo);
      Exit;
    End;
    If Not TR Then Inc(OnePass);
    If Not TR and (OnePass = 2) Then Fail := True;
  End;
  CloseIt;

End;  { Proc GetNext }

Procedure PrintSN;

Begin { Proc PrintSN }

  Window(1, 1, 80, 25); ClrScr;
  RomPtr := RomData;
  RomDta := RomPtr^;
  GotoXY(22, 1);
  Write('SERIAL NUMBER: [');
  For I := 7 downto 2 Do Write(Hex(RomDta[i]):3);
  Write(' ]');

End;  { Proc PrintSN }

Procedure DispData;

Begin { Proc DispData }

  RomPtr := RomData;
  RomDta := RomPtr^;
  GotoXY(22, 2);
  Write('SERIAL NUMBER: [');
  For I := 7 downto 2 Do Write(Hex(RomDta[i]):3);
  Write(' ]');
  GotoXY(11, 4);
  Write('REAL TIME CLOCK:');
  If ReadScratch(PageData, AuthCode) Then;
  PrintIt(3);
  If Read_S_Mem(PageNo, PageData) Then;
  PrintIt(4);

End;  { Proc DispData }

Var
  Ltemp, Savef : Array[0..3] of LongInt;

Procedure FormatTicks;

Begin { Proc FormatTicks }

  L := 0;
  For I := 0 to 3 Do L := L + (Ltemp[I] Shl (I Shl 3));
  UTD(L, Da_Date, Ti_Time);

End;  { Proc FormatTicks }

Procedure GetTicks;

Begin { Proc GetTicks }

  If Read_S_Mem(16, PageData) Then;                  { Read the clock page }
  AlarmF := PageData[0] and 7;
  For I := 0 to 3 Do Ltemp[I] := PageData[3 + I];
  FormatTicks;

End;  { Proc GetTicks }

Const
  DecSec : LongInt = 0;

Procedure UpDate;

Var
  OscOn  : Boolean;              { Flag set to true if the oscillator is on }

Begin { Proc UpDate }

  GetTicks;
  GotoXY(XM[0], YM[0]);
  If (L > DecSec) Then Begin
    Write(LZ(Ti_Time.Hour), ':', LZ(Ti_Time.Min), ':', LZ(Ti_Time.Sec));
    GotoXY(XM[0] + 25, YM[0]);
    Write(LZ(Da_Date.Mon), '/', LZ(Da_Date.Day), '/', Da_Date.Year);
  End Else Begin
    Write('00:00:00');
    GotoXY(XM[0] + 25, YM[0]);
    Write('00/00/0000');
  End;
  If PageNo = 16 Then PrintIt(4);
  If Boolean(Alarmf and 1) Then Begin
    GotoXY(1, YM[0] + 2); Center('A REAL TIME CLOCK ALARM HAS OCCURRED');
  End;
  If Boolean(Alarmf and 2) Then Begin
    GotoXY(1, YM[0] + 3); Center('AN ELAPSE TIMER ALARM HAS OCCURRED');
  End;
  If Boolean(Alarmf and 4) Then Begin
    GotoXY(1, YM[0] + 4); Center('A CYCLE COUNTER ALARM HAS OCCURRED');
  End;
  if Odd(PageData[1] Shr 4 and 1) then OscOn := True Else OscOn := False;
  If Tick and OscOn Then
  Begin                                                   { Tick if osc. on }
    Sound(5000); Delay(1); NoSound;
  End;
End;  { Proc UpDate }

Procedure PrintFld;

Begin { Proc PrintFld }

  If L >= DecSec Then
     Write(LZ(Ti_Time.Hour), ':', LZ(Ti_Time.Min), ':', LZ(Ti_Time.Sec), '     ',
           LZ(Da_Date.Mon), '/', LZ(Da_Date.Day), '/', Da_Date.Year)
  Else
     Write('00:00:00     00/00/0000');

End;  { Proc PrintFld }

Const
  SP : Array[0..3] of Byte = ( 3, 17, 8, 22 );

Procedure PrintClk;

Begin { Proc PrintClk }

  PrintSN;
  ComBar;
  Write(' LARR, RARR => EDITING     F1 => LEDGEND     ESC => QUIT');
  Textwin;
  ClrScr;
  GotoXY(15, 2); Write('      REAL TIME CLOCK:');
  GotoXY(15, 4); Write('REAL TIME CLOCK ALARM:');
  GotoXY(15, 6); Write('         ELAPSE TIMER:');
  GotoXY(15, 8); Write('   ELAPSE TIMER ALARM:');
  GotoXY(15, 10); Write('       CYCLE COUNTER:');
  GotoXY(15, 12); Write(' CYCLE COUNTER ALARM:');
  GotoXY(1, 13);
  For I := 1 to 80 Do Write('');
  GotoXY(1, 14); Center('CONFIGURATION/STATUS REGISTERS  (200H AND 201H)');
  GotoXY(6, 16); Write('DON''T CARE            READ ONLY');
  GotoXY(6, 17); Write('                         ');
  GotoXY(6, 18); Write(' Ŀ    Ŀ    STRT AUTO');
  GotoXY(6, 19);
  Write(' XXX XXX CCE ETE RTE CCF ETF RTF DSL STP MAN OSC RO  WPC WPE WPR');
  GotoXY(7, 20); For I := 0 to 63 Do Write('');
  GotoXY(7, 21);
  For J := 0 to 1 Do For I := 7 DownTo 0 Do          { Write current values }
    If (PageData[J] Shr I) and 1 = 1 Then
       Write('ON  ')
    Else
       Write('OFF ');
  For J := 0 to 3 Do
  Begin                                            { Print the clock fields }
    For I := 0 to 3 Do Ltemp[I] := PageData[I + SP[J]];
    FormatTicks;
    If L >= DecSec Then Savef[J] := L;
    GotoXY(40, 2 * (J + 1)); PrintFld;          { Position cursor and print }
  End;
  For I := 0 To 3 Do Ltemp[I] := PageData[I + 12];
  L := 0;
  For I := 0 To 3 Do L := L + (Ltemp[I] Shl (I Shl 3));
  GotoXY(40, 10); Write(L);                    { Print cycle counter value }
  For I := 0 TO 3 Do Ltemp[I] := PageData[I + 26];
  L := 0;
  For I := 0 To 3 Do L := L + (Ltemp[I] Shl (I Shl 3));
  GotoXY(40, 12); Write(L);              { Print cycle counter alarm value }

End;  { Proc PrintClk }

Const
  Ofst : Array[0..9] of Byte = ( 3, 3, 17, 17, 8, 8, 22, 22, 12, 26 );
  Max  : Array[0..9] of Byte = ( 6, 8, 6, 8, 6, 8, 6, 8, 10, 10 );
  XC   : Array[0..25] of Byte = ( 40, 53, 40, 53, 40, 53, 40, 53, 40, 40,
                                  7, 11, 15, 19, 23, 27, 31, 35, 39, 43,
                                  47, 51, 55, 59, 63, 67 );
  YC   : Array[0..25] of Byte = ( 2, 2, 4, 4, 6, 6, 8, 8, 10, 12, 21, 21,
                                  21, 21, 21, 21, 21, 21, 21, 21, 21 ,21,
                                  21, 21, 21, 21 );
Var
  PLC : Byte;               { Pointer to data array element for input data }
  KS  : Byte;                                    { Byte value of keystroke }

Procedure GetHexBytes(POS: Byte);

Begin { Proc GetHexBytes }

  For I := 0 to 3 Do Begin
    PageData[i + ofst[POS]] := L and $FF;
    L := L Shr 8;
  End;

End;  { Proc GetHexBytes }

Function POW(XPOW: Byte; YPOW: Byte): LongInt;

Var
  TempPow : LongInt;

Begin { Func POW }

  TempPow := 1;
  For I := 1 To YPOW Do TempPow := TempPow * 10;
  POW := TempPow;

End;  { Func POW }

Procedure ClockPage;

Const
  Fbit : Byte = 1;

Var
  Dbuff   : Array[0..9] of Byte;     { Temporary storage for keyboard input }
  POS     : Byte;
  Skip    : Boolean;
  Yword   : Word;
  Cvar    : Array[0..4] of Byte;
  Tstore  : Array[0..4] of LongInt;   { Temp. storage for long calculations }
  Charr   : Char;

Begin { Proc ClockPage }

  If Read_S_Mem(16, PageData) Then;                   { Read the clock page }
     PrintClk;                                       { Print the clock page }
  POS := 0;
  PLC := 0;
  GotoXY(XC[POS], YC[POS]);                 { Go to initial cursor position }
  While True Do
  Begin { -- Infinite While -- }

    Skip := False;
    KS := Byte(ReadKey);
    Case KS of
      0:
      Begin { -- Outer Case 0 -- }

        Case Byte(ReadKey) of
          77:
          Begin { -- Case 77 -- }
            If PLC = 0 Then
            Begin
              If POS = 14 Then POS := POS + 4
              Else If POS = 25 Then POS := 0 Else Inc(POS);
              GotoXY(XC[POS], YC[POS]);
            End;
          End;  { -- Case 77 -- }

          75:
          Begin { -- Case 75 -- }
            If PLC = 0 Then
            Begin
              If POS = 18 Then POS := POS - 4
              Else If POS = 0 Then POS := 25 Else Dec(POS);
              GotoXY(XC[POS], YC[POS]);
            End;
          End;  { -- Case 75 -- }

          72:
          Begin { -- Case 72 -- }
            GetNext;
            If Read_S_Mem(16, PageData) Then;
                PrintClk;
            POS := 0; PLC := 0;
            GotoXY(XC[POS], YC[POS]);
          End;  { -- Case 72 -- }

          80:
          Begin { -- Case 80 -- }
            GetLast;
            If Read_S_Mem(16, PageData) Then;
            PrintClk;
            POS := 0; PLC := 0;
            GotoXY(XC[POS], YC[POS]);
          End;  { -- Case 80 -- }

          59:
          Begin { -- Case 59 -- }
            Help;
            PrintClk;
            POS := 0;
            PLC := 0;
            GotoXY(XC[POS], YC[POS]);
          End;  { -- Case 59 -- }

          20:
          Begin { -- Case 20 -- }
            If WriteScratch(16, PageData) Then;
                If ReadScratch(PageData, AuthCode) Then;
                    AuthCode[2] := AuthCode[2] and $7F;
            For I := 0 to 2 Do
            Begin { -- For -- }
              If CopyScratch(AuthCode) Then
                 AuthCode[2] := AuthCode[2] or $80;
            End;  { -- For -- }

            Exit;
          End;  { -- Case 20 -- }
        End; { Inner CASE Statement }

      End;  { -- Outer Case 0 -- }

      27:
      Begin { -- Case 27 -- }
        Window(1, 1, 80, 25);
        ClrScr;
        Halt;
      End;  { -- Case 27 -- }

      8:
      Begin { -- Case 8 -- }
        If (PLC <> 0) and (POS < 10) Then Begin
          If POS < 8 Then
             If (PLC = 2) or (PLC = 4) Then
                GotoXY(WhereX - 1, WhereY);
          Write(Char(KS), ' ', Char(KS));
          Dec(PLC);
          Dbuff[PLC] := 0;
        End;
      End;  { -- Case 8 -- }

      13:
      Begin { -- Case 13 -- }
        If POS > 9 Then
        Begin { -- If -- }

          If POS < 18 Then
          Begin { -- If -- }

            PageData[0] := PageData[0] xor (Fbit Shl (17 - POS));
                                                                 { Comp Bit }
            If (PageData[0] Shr (17 - POS)) and 1 = 1 Then
               Write('ON ')
            Else
               Write('OFF');
            Inc(POS);
            If POS = 15 Then
               POS := POS + 3;

          End   { -- If -- }
          Else
          Begin { -- Else -- }

            PageData[1] := PageData[1] xor (Fbit Shl (25 - POS));
            If (PageData[1] Shr (25 - POS)) and 1 = 1 Then
               Write('ON ')
            Else
               Write('OFF');
            If Pos > 22 Then
            Begin
               Warning;
               ClrScr; GotoXY(30, 1); Write('ARE YOU SURE');
               Repeat
                  Charr := ReadKey;
               Until (UpCase(Charr) = 'N') or (UpCase(Charr) = 'Y');
               If UpCase(Charr) = 'N' then
               Begin
                  PageData[1] := PageData[1] xor (Fbit Shl (25 - POS));
                  If (PageData[1] Shr (25 - POS)) and 1 = 1 Then
                     Write('ON ')
                  Else
                     Write('OFF');
               End;
            End;
            PrintClk;
            If POS = 25 Then
               POS := 0
            Else
               Inc(POS);

          End;  { -- Else -- }
          GotoXY(XC[POS], YC[POS])

        End  { -- If -- }
        Else
        Begin { -- Else -- }

          If (POS and 1 = 0) and (POS <> 8) and (PLC = MAX[POS]) Then
          Begin
            For I := 0 To 2 Do
                Cvar[I] := Dbuff[I Shl 1]*10 + Dbuff[(I Shl 1) + 1];
            If (Cvar[0] < 24) and (Cvar[1] < 60) and (Cvar[2] < 60) Then
            Begin
              If (Savef[POS Shr 1] > DecSec) Then
                 UTD(Savef[POS Shr 1], Da_Date, Ti_Time);
              Ti_Time.Hour := Cvar[0];
              Ti_Time.Min := Cvar[1];
              Ti_Time.Sec := Cvar[2];
              L := DTU(Da_Date, Ti_Time);
              Savef[POS Shr 1] := L;
              GetHexBytes(POS);
              PLC := 0; Inc(POS); GotoXY(XC[POS], YC[POS]); { Set new pos. }
            End
            Else
            Begin
              PLC := 0; GotoXY(XC[POS], YC[POS]);
            End;
          End
          Else If (POS and 1 = 1) and (POS <> 9) and (PLC = MAX[POS]) Then
          Begin
            For I := 0 to 2 Do
                Cvar[I] := Dbuff[I Shl 1]*10 + Dbuff[(I Shl 1) + 1];
            Yword := Dbuff[4]*1000 + Dbuff[5]*100 + Dbuff[6]*10 + Dbuff[7];
            If (Cvar[0] < 13) and (Cvar[1] < 32) and (Yword > 1969) Then
            Begin
           (*   If Savef[(POS - 1) Shr 1] > DecSec Then
                UTD(Savef[(POS - 1) Shr 1], Da_Date, Ti_Time); *)
              Da_Date.Year := Yword;
              Da_Date.Mon := Cvar[0];
              Da_Date.Day := Cvar[1];
              L := DTU(Da_Date, Ti_Time);
              Savef[(POS - 1) Shr 1] := L;
              GetHexBytes(POS);
              PLC := 0; Inc(POS); GotoXY(XC[POS], YC[POS]); { Set new pos. }
              If POS = 8 Then
                 Skip := True
              Else
                 Skip := False;
            End
            Else
            Begin
              PLC := 0; GotoXY(XC[POS], YC[POS]);
            End;
          End Else If Not Skip and (POS > 7) Then
          Begin
            L := 0;
            For J := 0 To PLC - 1 Do
            Begin
              Tstore[0] := Dbuff[J];
              L := L + Tstore[0] * POW(10, PLC - J - 1);
            End;
            GetHexBytes(POS);
            PLC := 0; Inc(POS); GotoXY(XC[POS], YC[POS]);
          End;

        End;  { -- Else -- }

      End;  { -- Case 13 -- }

      Else
      Begin { -- Outer CASE Default -- }

        If POS < 10 Then
        Begin { -- If -- }

            If PLC = 0 Then
            Begin
                If MAX[POS] = 10 Then Write('          ');
                If MAX[POS] = 8 Then Write('  /  /    ');
                If MAX[POS] = 6 Then Write('  :  :  ');
                GotoXY(XC[POS], YC[POS]);
            End;
            If (KS > 47) and (KS < 58) Then
            Begin
                If PLC <> MAX[POS] Then
                Begin
                   Dbuff[PLC] := KS - 48;
                   Inc(PLC);
                   Write(Char(KS));
                   If POS < 8 Then
                      If (PLC = 2) or (PLC = 4) Then
                         GotoXY(WhereX + 1, WhereY);
                End;
            End;

        End;  { -- If -- }

      End;  { -- Outer CASE Default -- }

    End; { -- Outer CASE Statement -- }

  End;  { -- Infinite While -- }

End;  { Proc ClockPage }

Const
  XH : Byte = 58;
  YH : Byte = 13;

Var
  PN        : Byte;                                                { Lpt # }
  XP, YP    : Byte;                 { Used to save current cursor location }
  KSarr     : Array[0..31] of Byte;          { Data received from keyboard }
  LI        : LongInt;
  Z         : Word;
  TempSec   : Byte;                             { Save current second byte }
  Doit      : Boolean;     { If Doit is True then update clock information }
  PartCount : Boolean;
  PFound    : Boolean;
  Hr, Mnt, Second, Hsecond : Word;                     { Words for GetTime }
  Yr, Mont, Dy, DofW       : Word;                     { Words for GetDate }
  K                        : Boolean;
Begin { Main Program }

 (*
  PN := 1;                                             { Default to Lpt #1 }
  If ParamCount > 0 Then
  Begin

    {$R+}
    Val(ParamStr(1), LI, Z);                 { Get Lpt # from command line }
    {$R-}
    If (Z > 0) or (LI < 1) or (LI > 3) Then
    Begin
      WriteLn('INCORRECT COMMAND LINE PARAMETER');
      Halt;
    End
    Else
        PN := LI;

  End;

  K := DowCheck;
  If Not Setup(PN) Then
  Begin                  { Does port Lpt #PN exist ? }
    WriteLn('SPECIFIED PRINTER PORT DOES NOT EXIST');
    Halt;
  End;

  K := KeyOpen;                                {Get Control of Port}
  
  If Not GndTest Then
  Begin                        { Is line 13 grounded ? }
    Writeln('PRINTER NOT ON LINE');
    Halt;
  End;
  K := KeyClose;
 *)

  PFound := False;
  PN := 0;                                     { Search ports for DOW parts }
  While ((Not PFound) And (PN < 4)) do
  Begin
     Inc(PN);
     If Setup(PN) then
        If KeyOpen then
           If GndTest then
              If First then
                 PFound := True;
     K := KeyClose;
  End;

  If Not PFound then
  Begin
     WriteLn('No Buttons found on any LPT port\n');
     Halt;
  End; 
  
  K := KeyOpen;                                       { Get Control of Port }

  While Next Do
        ;
  GetNext;
  PartCount := True;

  K := KeyClose;                           { Release control between passes }
  K := KeyOpen;                                       { Get Control of Port }

  CurrNo := 1;
  PageNo := 0;
  If PartCount Then
  Begin                          { Begin main loop if a DOW part is present }
    InitCrt;                                           { Set up demo screen }
    DispData;                                { Display data in first DS1994 }
    GetTime(Hr, Mnt, Second, Hsecond);               { Get current DOS time }
    TempSec := Second;                                  { Save second value }
    Doit := True;                 { Doit = True => Update clock information }
    GotoXY(XH, YH);                        { Set cursor to initial location }
    PLC := 0;                       { Initialize keyboard input data buffer }
    While True Do
    Begin

      K := KeyClose;                       { Release Control between passes }
      K := KeyOpen;                                   { Get Control of port }

      If KeyPressed Then Begin
        KS := Byte(ReadKey);           { Get ascii value of keystroke in KS }
        Case KS of
          27:                   { Escape key pressed, clear screen and quit }
          Begin { -- Outer Case 27 -- }
            Window(1, 1, 80, 25);
            ClrScr;
            K := KeyClose;                    { Release Control before Exit }
            Halt;
          End;  { --  Outer Case 27 -- }

          8:                               { Backspace key has been pressed }
          Begin { --  Outer Case 8 -- }
            If Plc <> 0 Then
            Begin                                  { Don't backspace to far }
              Dec(PLC);                     { Point to last element entered }
              KSarr[PLC] := 32;              { Set to ascii value for space }
              If Plc = 15 Then
                 GotoXY(74, 13);                        { Go up to top line }
              Write(Char(KS), ' ', Char(KS));    { Send backspace to screen }
            End;
          End;  { --  Outer Case 8 -- }

          13:
          Begin { --  Outer Case 13 -- }
            If (PageNo <> 16) And Boolean(Plc) Then
            Begin
              For I := 0 to 31 Do PageData[I] := KSarr[I];  { Move new data }

              If WriteScratch(PageNo, PageData) Then
                 ;                                    {Send to scratch page }
              If ReadScratch(PageData, AuthCode) Then
                 ;                                      { Read scratch page }
              If CopyScratch(AuthCode) Then
                 ;                                { Copy into secure memory }
              If Read_S_Mem(PageNo, PageData) Then
                 ;                                   { Read data in s. mem. }

              Printit(3); PrintIt(4);    { Print Scratch page, s. mem. page }
              PLC := 0;                               { Reset array pointer }
              GotoXY(XH, YH);                       { Reset cursor position }
            End;
          End;  { -- Outer Case 13 -- }
          0:
          Begin { -- Outer Case 0 -- }          { Handle Extended Key Codes }

            Case Byte(ReadKey) of
              81:                                   { Page down key pressed }
              Begin { -- Case 81 -- }
                If PageNo = 0 Then
                   PageNo := 16
                Else
                   Dec(PageNo);
                XP := WhereX; YP := WhereY;          { Save cursor position }
                If Read_S_Mem(PageNo, PageData) Then
                   ;                                        { Read new page }
                PrintIt(4);                         { Print new page number }
                GotoXY(XP, YP);          { Restore original cursor position }
              End;  { -- Case 81 -- }

              73:                                     { Page up key pressed }
              Begin { -- Case 73 -- }
                If PageNo = 16 Then
                   PageNo := 0
                Else
                   Inc(PageNo);
                XP := WhereX; YP := WhereY;          { Save cursor position }
                If Read_S_Mem(PageNo, PageData) Then
                   ;                                        { Read new page }
                Printit(4);                         { Print new page number }
                GotoXY(XP, YP);          { Restore original cursor position }
              End;  { -- Case 73 -- }

              59:                                          { F1 key pressed }
              Begin { -- Case 59 -- }
                Help;                                    { Print key legend }
                InitCrt;                      { Restore demo display screen }
                DispData;                          { Restore data on screen }
                PLC := 0;                             { Reset place counter }
                GotoXY(XH, YH);   { Go to the beginning of the scratch page }
              End;  { -- Case 59 -- }

              80:
              Begin { -- Case 80 -- }
                GetLast;
                InitCrt;
                GetTicks;
                DispData;
                GetTime(Hr, Mnt, Second, HSecond);
                TempSec := Second;
                Doit := True;
                GotoXY(58, 13);
              End;  { -- Case 80 -- }

              72:
              Begin { -- Case 72 -- }
                GetNext;
                InitCrt;
                GetTicks;
                DispData;
                GetTime(Hr, Mnt, Second, HSecond);
                TempSec := Second;
                Doit := True;
                GotoXY(58, 13);
              End;  { -- Case 72 -- }

              20:
              Begin { -- Case 20 -- }
                ClockPage;                        { Enter clock page screen }
                PageNo := 16;
                InitCrt;                              { Reset memory screen }
                DispData;             { Display scratch, secure memory page }
                PLC := 0;                             { Reset array pointer }
                GotoXY(XH, YH);   { Go to the beginning of the scratch page }
              End;  { -- Case 20 -- }

            End; { -- END Inner Case -- }

          End;  { -- Outer Case 0 -- }          { Handle Extended Key Codes }

        Else { -- Outer Default Case -- }

          If PLC = 0 Then
          Begin
            For I := 0 to 31 Do
            Begin                                             { Clear field }
              KSarr[I] := 32;
              If I = 16 Then
                 GotoXY(XH, YH + 1);                      { Go to next line }
              Write(' ');
            End;
            GotoXY(XH, YH);                                  { Reset cursor }
          End;
          If PLC <> 32 Then
          Begin                                { Past max number of bytes ? }
            Write(Char(KS));                { Write character out to screen }
            KSarr[PLC] := KS;                  { Store ascii value in KSarr }
            Inc(PLC);                             { Increment array pointer }
            If PLC = 16 Then GotoXY(XH, YH + 1);          { Go to next line }
          End;

        End;  { -- Outer Default Case -- }

      End;  { -- END Outer Case --  }

      If Doit Then
      Begin                                       { Time to update screen ? }
         XP := WhereX; YP := WhereY;      { Save current screen coordinates }
         Update;                                { Read time from the DS1994 }
         GotoXY(XP, YP);                          { Restore cursor position }
         Doit := False;          { Do not want to update for another second }
      End
      Else
      Begin
         GetTime(Hr, Mnt, Second, Hsecond);             { Get time from DOS }
         If Second <> TempSec Then
         Begin                                       { Has second changed ? }
           Doit := True;               { Update clock information next pass }
           TempSec := Second;                             { Save new second }
         End;
      End;
    End;
  End
  Else
     WriteLn('ATLEAST ONE DS1994 MUST BE CONNECTED');            { No DOW's }

  K := KeyClose;

End.  { Main Program }

