/* ************************************************************************* */
/* ProgTime.c                                                                */
/* DS Software Authorization SDK 4.3                                         */
/* Disk 1 - Dos Object & Source Files                                        */
/* 02/13/98                                                                  */
/*                                                                           */
/* Copyright (C) 1996 Dallas Semiconductor Corporation.                      */
/* All rights Reserved. Printed in U.S.A.                                    */
/* This software is protected by copyright laws of                           */
/* the United States and of foreign countries.                               */
/* This software is furnished under a license agreement and/or a             */
/* nondisclosure agreement and may only be used or copied in accordance      */
/* with the terms of those agreements.                                       */
/* The mere transfer of this software does not imply any licenses            */
/* of trade secrets, proprietary technology, copyrights, patents,            */
/* trademarks, maskwork rights, or any other form of intellectual            */
/* property whatsoever. Dallas Semiconductor retains all ownership rights.   */
/*                                                                           */
/* Additional Notes:                                                         */
/*                                                                           */
/*    This code was compiled using Boreland 4.5 along with the Turbo         */
/*    Assembler.  Use of other compilers may require slight modification.    */
/*                                                                           */
/*    In order to make the project, link this file with either Sauth400.c,   */
/*    or Sauth400.asm.                                                       */
/*                                                                           */
/*    This software is intended for use with DS1410D and DS1410E and later   */
/*    model parallel port adapters only.                                     */
/*                                                                           */
/*    This program searches for a DS1994. If found it sets the RTC (clock)   */
/*    and turns on the oscillator bit. It will also read the first page of   */
/*    RAM and Prompts for a string. The new data will be written to the      */
/*    first page.                                                            */
/*                                                                           */
/* ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <mem.h>

#define TRUE  1
#define FALSE 0

typedef unsigned char  uchar;
typedef unsigned long  ulong;
typedef unsigned short ushort;

typedef struct _ScratchPad
{
   unsigned char authcode[3];
   unsigned char scratchdata[32];
}
ScratchPad, *PScratchPad;

typedef struct _ClockRegs
{
   uchar StatusBytes[2];  // Configuration bytes
   uchar SecDiv256;       // 256's of a second (LSB of RTC)
   uchar BTime[4];        // 4 MSB's of Real Time Clock
   uchar OtherRegs[25];   // Other registers in the clock page 
}
ClockRegs, *PClockRegs;

//
//   SecureTMEX (access system) function prototypes
//
uchar far pascal dowcheck(void);  
uchar far pascal keyopen(void);
uchar far pascal keyclose(void);
uchar far pascal setup(uchar);
uchar far pascal next(void);
uchar far pascal first(void);
uchar far pascal access(void);
uchar far pascal gndtest(void);
uchar far pascal databyte(uchar);
uchar far * far pascal romdata(void);

ClockRegs  ClockPage;
ScratchPad ScratchPage;
ushort     pa_vals[3] = { 0x378, 0x278, 0x3BC };

uchar ReadRAM(uchar *RAMBuffer, uchar PageNum)
{
   int   i;

   // Reset/Address button 
   if (access())
   {
      // Send read secure memory
      databyte(0xF0);
      // Send low byte of target address for PageNum
      databyte(PageNum << 5);
      // Send high byte of target address for PageNum
      databyte(PageNum >> 3); 

      // Read all 32 byte of RAM dat from page #PageNum
      for (i = 0; i < 32; i++)
         *(RAMBuffer + i) = databyte(0xFF);

      return TRUE;
   }

   return FALSE;
}

uchar SetTime(void)
{
   ulong SecSince70;
   uchar PageNum;
   int   i;

   // Clock page is last page of memory (counting from 0!!!)
   PageNum = 16;

   // Reset/Address button 
   if (ReadRAM(ClockPage.StatusBytes, PageNum))
   {
      // Get number of seconds since 1970
      SecSince70 = time(NULL);

      // Shift long value into bytes
      for (i = 0; i < 4; i++)   
      {
         ClockPage.BTime[i] = (uchar) (SecSince70 & 0xFF);
         SecSince70 >>= 8;
      }

      // Make sure oscillator bit is (1) on
      ClockPage.StatusBytes[1] |= 0x10;

      if (access())
      {
         // Send write scratchpad command byte
         databyte(0x0F);
         // Send low byte of target address
         databyte(PageNum << 5);
         // Send high byte of target address
         databyte(PageNum >> 3);
   
         // Write data to scratchpad
         for (i = 0; i < 32; i++)
            databyte(*(ClockPage.StatusBytes + i));

         if (access())
         {
            // Send the copy scratchpad command byte
            databyte(0x55);
            // Send the 1st byte of the authorization code
            databyte(PageNum << 5);
            // Send the 2nd byte of the authorization code
            databyte(PageNum >> 3);
            // Send the 3rd byte of the authorization code (ending address)
            databyte(0x1F);

            return TRUE;
         }
      }
   }

   return FALSE;
}

uchar ReadTime(ulong *RTCVal)
{
   uchar PageNum;
   int   i;

   // Page number of clock registers
   PageNum = 16;
   // Initialize RTC value
   *RTCVal = 0;

   if (access())
   {
      // Send read secure memory
      databyte(0xF0);
      // Send low byte of target address for PageNum
      databyte(PageNum << 5);
      // Send high byte of target address for PageNum
      databyte(PageNum >> 3); 

      // We don't care about the first three bytes in this function
      databyte(0xFF); databyte(0xFF); databyte(0xFF); 

      // Read all 4 most significant RTC bytes 
      for (i = 0; i < 4; i++)
         *RTCVal |= (((ulong) databyte(0xFF)) << (i << 3));

      return TRUE;
   }

   return FALSE;
}

uchar WriteRAM(uchar *RAMBuffer, uchar PageNum)
{
   int i;

   // Reset/Address button 
   if (access())
   {
      // Send write scratchpad command
      databyte(0x0F);
      // Send low byte of target address
      databyte(PageNum << 5);
      // Send high byte of target address
      databyte(PageNum >> 3);

      // Write data in RAMBuffer to DS1994 scratchpad
      for (i = 0; i < 32; i++)
         databyte(*(RAMBuffer + i));

      // Reset DS1994
      if (access())
      {
         // Send the copy scratchpad command byte
         databyte(0x55);
         // Send the 1st byte of the authorization code
         databyte(PageNum << 5);
         // Send the 2nd byte of the authorization code
         databyte(PageNum >> 3);
         // Send the 3rd byte of the authorization code (ending address)
         databyte(0x1F);

         return TRUE;
      }
   }

   return FALSE;
}

void Intro(void)
{
   printf("\n               ProgTime.c - sets the DS1994's clock to the");
   printf("\n               system time. After setting the clock it reads");
   printf("\n               the clock and displays the results as a"); 
   printf("\n               confirmation. Next it asks for data to write");
   printf("\n               into the first (of sixteen) RAM page. After");
   printf("\n               writing this data it reads the first RAM page"); 
   printf("\n               and displays the results as a confirmation"); 
   printf("\n\n               Note that this simple program only uses scanf");
   printf("\n               for input. Don't use spaces\n\n"); 
}

void main(void)
{
   uchar pn = 0;       // Parallel port number 
   uchar far *addr;    // Holds ROM data pointer 
   uchar RamPage0[32]; // RAM data buffer
   uchar TempStr[80];
   uchar Found1994;
   ulong TimeVal;      // 32 bit value for RTC
   int   i;

   Intro();

   memset(RamPage0, 0, 32);

   // See if device driver is installed
   if (dowcheck())                
   {
      // Search all ports
      while ((++pn < 4) && setup(pn))                   
      {
         // Gain control of port access
         keyopen();
         
         // Make sure we can talk to DS1410 
         if (gndtest())               
         {
            // Look for first part on bus
            if (first())                
            {
               Found1994 = FALSE;

               do 
               {
                  // Release control of port between passes 
                  keyclose();  
                  // Regain control of port
                  keyopen();
   
                  // Get pointer to ROM data 
                  addr = romdata();

                  // Is this a DS1994 time button
                  if (((*addr == 0x84) || (*addr == 0x04)) && access())
                  {
                     Found1994 = TRUE;

                     printf("\nDS1994 Rom Data: ");         
                     for (i = 0; i < 8; i++)    
                        printf("%2.2X ", *(addr + i));

                     TimeVal = time(NULL);
                     printf("\nSetting DS1994's clock to system time : %s",
                            ctime(& (long)TimeVal));

                     // Set the RTC in this DS1994
                     if (!SetTime())
                     {
                        printf("Unable to set time in DS1994\n");
                        exit(1);
                     }

                     // Read back RTC value
                     if (!ReadTime(&TimeVal))
                     {
                        printf("Unable to read back time from DS1994\n");
                        exit(1);
                     }
                     else
                     {
                        printf("Time read from DS1994 : %s\n", 
                               ctime(& (long)TimeVal));
                     }

                     memset(TempStr, 0x00, 80);
                     printf("New string for first RAM page (<= 32 bytes) :");
                     scanf("%s", TempStr);
                     memcpy(RamPage0, TempStr, 31);  
                     RamPage0[31] = 0;  
                    
                     // Write new RAM data to DS1994
                     if (!WriteRAM(RamPage0, 0))
                     {
                        printf("Unable to write DS1994's RAM\n");
                        exit(1);
                     }

                     // Get contents of first page of RAM
                     if (!ReadRAM(RamPage0, 0))
                     {
                        printf("Unable to write DS1994's RAM\n");
                        exit(1);
                     }
                     else
                     {
                        i = 0;

                        printf("Reading back data in first RAM page         :");
                        // Display data read from first RAM page
                        while (RamPage0[i] && (i < 32))
                           putchar(RamPage0[i++]);

                        printf("\n");
                     }
                  }
               }
               while (next()); // Look at all parts on port

               if (!Found1994)
                  printf("Unable to find any DS1994/DS1427s in DS1410\n");
            }
            else 
            {
               printf("Unable to find DS1410 on port address %4.4X\n", 
                      pa_vals[pn - 1]);
            }
         } 
         else 
         {
            printf("Unable to find DS1410 on port address %4.4X\n", 
                   pa_vals[pn - 1]);
         }
         // Release control of access system 
         keyclose();
      }

   }
   else
   {
      // Not meaningful in DOS, used for portability
      printf("DOW device driver not installed.\n");
   }
}


